import { defineStore, type StoreDefinition } from 'pinia'
import { io, Socket } from 'socket.io-client';
import config from '@/config';

export const useSocketStore: StoreDefinition<any> = defineStore('socket', {
  state: () => ({
    socket: null as Socket | null,
    isAuthenticated: false,
    character: null as any,
  }),
  getters: {
    getSocket: (state: any) => state.socket,
    getAuthenticated: (state: any) => state.isAuthenticated,
    getCharacter: (state: any) => state.character,
  },
  actions: {
    setupSocketConnection(username: string, password: string) {
      this.socket = io(config.server_endpoint, {
        query: { username, password },
        transports: ['websocket']
      });
    },
    disconnectSocket() {
      if (!this.socket) {
        return;
      }

      this.socket.disconnect();
      this.socket = null;
      this.character = null;
      this.isAuthenticated = false;
    },
  }
});

/**
 * Resources:
 * https://www.dynetisgames.com/2017/03/06/how-to-make-a-multiplayer-online-game-with-phaser-socket-io-and-node-js/
 * https://runthatline.com/pinia-watch-state-getters-inside-vue-components/
 * https://pinia.vuejs.org/
 */